package com.unis.sxls.server.controller;

import com.unis.sxls.server.model.FileInfoVO;
import com.unis.sxls.server.repository.FileInfoRepository;
import com.unis.sxls.server.server.FileInfoServer;
import com.unis.sxls.server.untils.FilesUtils;
import com.unis.sxls.server.untils.POIReadExcelToHtml;
import com.unis.sxls.server.untils.ResponseWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import tk.mybatis.mapper.entity.Example;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.File;
import java.io.OutputStream;
import java.util.*;


/**
 * @author  yang yunhai
 * 文件上传下载处理层
 * @date : 时间:2018/6/14 9:36
 */
@RestController
@RequestMapping("/file")
@Slf4j
@Api(value = "文件上传下载",tags = "文件上传下载处理层")
public class FileInfoController {

    @Autowired
    private FilesUtils filesUtils;
    @Autowired
    private FileInfoRepository fileInfoRepository;
    @Autowired
    private POIReadExcelToHtml poiReadExcelToHtml;
    @Autowired
    private FileInfoServer fileInfoServer;


    /**
     * @return com.unis.sxls.portal.untils.ResponseWrapper
     * @author yangyh
     * 文件上传
     * @date 2018/6/28 18:25
     */
    @ApiOperation(value = "上传文件", notes = "参数：文件")
    @RequestMapping(value = "/upload",method = RequestMethod.POST, consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public ResponseWrapper upload(
            HttpServletRequest request,
            @RequestBody MultipartFile file,
            @RequestParam(value = "businessId", required = false) String businessId
    ) {
        try {
            long size = file.getSize();
            long wjdx = size/1024000;
            Map<String, Object> result = null;
            if(wjdx<=500) {
                result  = fileInfoServer.executeUpload(file, request, businessId);
            }else {
                return ResponseWrapper.success( 412, "文件过大，文件不允许超过500M");
            }
            return ResponseWrapper.success( 200, "文件上传成功", result);
        } catch (Exception e) {
            e.printStackTrace();
            log.error( "上传文件，发生异常", e.getMessage() );
            return ResponseWrapper.error( 412, "文件上传失败！" );
        }
    }

    @ApiOperation(value = "上传缩略图", notes = "参数：缩略图文件")
    @ApiImplicitParams({@ApiImplicitParam(name = "nailType", value = "nailType", required = true)})
    @RequestMapping(value = "/uploadThumbnail/{nailType}",method = RequestMethod.POST, consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public ResponseWrapper uploadThumbnail(@PathVariable("nailType") String nailType, @RequestBody MultipartFile file) {
        try {
            Map<String, Object> result = fileInfoServer.uploadThumbnail(nailType, file);
            return ResponseWrapper.success( 200, "文件上传成功", result);
        } catch (Exception e) {
            log.error( "上传文件，发生异常", e.getMessage() );
            return ResponseWrapper.error( 412, "文件上传失败！" );
        }
    }

    /**
     * @param id
     * @author yangyh
     * @Description: 文件下载
     * @date 2018/6/28 18:25
     */
    @ApiOperation(value = "下载文件", notes = "参数：文件服务器地址")
    @GetMapping(value = "/download")
    public ResponseWrapper download(@RequestParam("id") Long id, HttpServletResponse response) {

        try {
            FileInfoVO fileInfoVO = fileInfoRepository.findById(id).get();
            if( ObjectUtils.equals(fileInfoVO, null) ){
                return ResponseWrapper.error( 404, "文件未找到，下载失败！" );
            }

            String originalFilename = fileInfoVO.getName();
            String path = fileInfoVO.getLink();
            filesUtils.downloadFile(originalFilename, path, response);
//            return ResponseWrapper.success( 200, "下载成功", path );
            return null;
        } catch (Exception e) {
            e.printStackTrace();
            log.error( "下载对象，发生异常", e.getMessage() );
            return ResponseWrapper.error( 500, "下载失败" );
        }

    }
    
	@ApiOperation(value = "返回FILE对象", notes = "参数：文件id")
    @GetMapping(value = "/downloadId")
    public File download(@RequestParam("id") Long id) {
    	
    	try {
    		FileInfoVO fileInfoVO = fileInfoRepository.findById(id).get();
    		 String originalFilename = fileInfoVO.getName();
    		String path = fileInfoVO.getLink();
    		 File file = filesUtils.download(originalFilename, path);
    		return file;
    	} catch (Exception e) {
    		e.printStackTrace();
    		log.error( "下载对象，发生异常", e.getMessage() );
    		return null;
    	}
    }


    /**
     * @param id
     * @author YangZR
     * @Description: 文件删除
     * @date 2018/6/28 18:25
     */
    @ApiOperation(value = "文件删除", notes = "参数：文件ID")
    @GetMapping(value = "/delete")
    public ResponseWrapper delete(@RequestParam("id") Long id) {
        try {
            Optional<FileInfoVO> optional = fileInfoRepository.findById(id);
            if(optional.isPresent()) {
                FileInfoVO fileInfoVO = optional.get();
                if( ! ObjectUtils.equals(fileInfoVO, null) ){
                    fileInfoRepository.deleteById(id);
                    filesUtils.deleteFile(fileInfoVO.getLink());
                    return ResponseWrapper.success( 200, "删除成功");
                }
            }
            return ResponseWrapper.error(403, "不存在该文件，删除失败！");
        } catch (Exception e) {
            e.printStackTrace();
            log.error( "下载对象，发生异常", e.getMessage() );
            return ResponseWrapper.error( 500, "下载失败" );
        }

    }

    /**
     * @param id
     * @return com.unis.sxls.portal.untils.ResponseWrapper
     * @author yangyh
     * @Description: 查询文件对象
     * @date 2018/6/28 18:25
     */
    @ApiOperation(value = "查询文件信息", notes = "参数：文件id")
    @PostMapping(value = "/findById")
    public ResponseWrapper findById(@RequestParam("id") Long id) {
        try {
            FileInfoVO fileInfoVO = fileInfoRepository.findById(id).get();
            if(ObjectUtils.equals(fileInfoVO,null)){
                return ResponseWrapper.error( 304, "未找到正确对象！" );
            }
            return ResponseWrapper.success( 200, "查询成功", fileInfoVO );
        } catch (Exception e) {
            log.error( "查询失败，发生异常", e.getMessage() );
            return ResponseWrapper.error( 500, "查询失败" );
        }
    }

    /**
     * @param ids
     * @return com.unis.sxls.portal.untils.ResponseWrapper
     * @author yangyh
     * @Description: 查询文件对象信息集合
     * @date 2018/6/28 18:25
     */
    @ApiOperation(value = "查询文件信息集合", notes = "参数：文件id集合")
    @GetMapping(value = "/findByIds")
    public ResponseWrapper findByIds(@RequestParam("ids") List<Long> ids) {
        try {
            List<FileInfoVO> fileInfoVOS = fileInfoRepository.findByIdIn(ids);
            if (ObjectUtils.equals(fileInfoVOS, null)) {
                return ResponseWrapper.error(304, "未找到正确对象！");
            }
            return ResponseWrapper.success(200, "查询成功", fileInfoVOS);
        } catch (Exception e) {
            log.error("查询失败，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "查询失败");
        }
    }

    /**
     * @param
     * @return com.unis.sxls.portal.untils.ResponseWrapper
     * @author yangyh
     * @Description: 查询文件对象信息集合
     * @date 2018/6/28 18:25
     */
    @ApiOperation(value = "查询文件信息集合", notes = "参数：文件id集合 例如:123,223,555,55")
    @GetMapping(value = "/findByAll/{id}")
    public ResponseWrapper findByAll(@PathVariable("id") String id) {
        try {
            List<Long> list=new ArrayList();
            String[] ids=id.split(",");
            for(int i=0;i<ids.length;i++){
                list.add(Long.valueOf(ids[i]));
            }
            List<FileInfoVO> fileInfoVOS = fileInfoRepository.findByIdIn(list);
            if (ObjectUtils.equals(fileInfoVOS, null)) {
                return ResponseWrapper.error(304, "未找到正确对象！");
            }
            return ResponseWrapper.success(200, "查询成功", fileInfoVOS);
        } catch (Exception e) {
            log.error("查询失败，发生异常", e.getMessage());
            return ResponseWrapper.error(500, "查询失败");
        }
    }




    /**
     * 预览Excel文件
     * @param id
     * @param response
     * @date 2018/8/31
     * @return com.unis.sxls.server.untils.ResponseWrapper
     */
    @ApiOperation(value = "预览Excel文件", notes = "参数：文件id")
    @PostMapping(value = "/excelToPage")
    public ResponseWrapper excelToPage(@RequestParam("id") Long id) {
        try {
            FileInfoVO fileInfoVO = fileInfoRepository.findById(id).get();
            if(ObjectUtils.equals(fileInfoVO,null)){
                return ResponseWrapper.error( 304, "未找到正确对象！" );
            }

            String realLink = fileInfoVO.getLink();
            if(StringUtils.isBlank(realLink)){
                return ResponseWrapper.error( 304, "未找到文件保存地址！" );
            }
            String  html = poiReadExcelToHtml.excelToHtml(realLink);
            return ResponseWrapper.success( 200, "查询成功", html );
        } catch (Exception e) {
            log.error( "查询失败，发生异常", e.getMessage() );
            return ResponseWrapper.error( 500, "查询失败" );
        }
    }

    /**
     * @param businessId
     * @author qss
     * @Description: 文件下载
     * @date 2022/8/24
     */
    @ApiOperation(value = "下载文件", notes = "参数：文件服务器地址")
    @GetMapping(value = "/downloadZip")
    public ResponseWrapper downloadZip(@RequestParam("businessId") String businessId, HttpServletResponse response) {

        try {
            List<FileInfoVO> fileList = fileInfoRepository.findByBusinessIdAndStatus(businessId,1);
            if(fileList.isEmpty()){
                return ResponseWrapper.error( 404, "文件未找到，下载失败！" );
            }

            String fileName = "文件";
            filesUtils.downloadFileZip(fileName, fileList, response);
            return null;
        } catch (Exception e) {
            e.printStackTrace();
            log.error( "下载对象，发生异常", e.getMessage() );
            return ResponseWrapper.error( 500, "下载失败" );
        }

    }

}
