package com.unis.sxls.server.server;


import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.transaction.Transactional;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import com.unis.sxls.server.model.FileInfoVO;
import com.unis.sxls.server.repository.FileInfoRepository;
import com.unis.sxls.server.untils.DateUtil;
import com.unis.sxls.server.untils.FilesUtils;
import com.unis.sxls.server.untils.IdWorker;
import tk.mybatis.mapper.entity.Example;

/**
 * @author  yang yunhai
 * 文件上传下载处理层
 * @date : 时间:2018/6/14 9:36
 */
@Service
public class FileInfoServer {

    /**
     * util
     */
    @Autowired
    private FilesUtils filesUtils;

    /**
     * repository
     */
    @Autowired
    private FileInfoRepository fileInfoRepository;
    
    @Autowired
	private IdWorker idWorker;
    
    /**
     * 执行文件上传操作
     * @param file
     * @param request
     */
    @Transactional
    public Map<String,Object> executeUpload(MultipartFile file, HttpServletRequest request, String businessId){

        String nameStr= "";
        String yearMonth = DateUtil.getNowDateFormat(DateUtil.YEAR_MONTH);
//        nameStr = CurrentUtilsLSJ.getUserInfo(CurrentUtilsLSJ.USER_NAME);
//        if(StringUtils.isBlank(nameStr)){
//            nameStr = CurrentUtils.getUserInfo(CurrentUtils.USER_NAME);
//        }
        //
        String fileUri = filesUtils.upload(file, yearMonth);
        if(StringUtils.isBlank(fileUri)){
            throw new RuntimeException("文件上传失败");
        }
        //
        FileInfoVO info = new FileInfoVO();
        System.out.println("file.getOriginalFilename()==" + file.getOriginalFilename());

        if (request.getHeader("user-agent").toLowerCase().contains("msie")) {
            // IE
          String   filename = getFileName(file.getOriginalFilename());
            info.setName(filename);
        } else {
            // 非IE
            info.setName(file.getOriginalFilename());
        }

        System.out.println("上传文件名称======="+info.getName());


     // 生成Id
		long id = idWorker.nextId();
		info.setId(id);
        info.setFileSize(file.getSize());
//        info.setCreateByPeople(nameStr);
        info.setCreateTime(new Date());
        info.setLink(fileUri);
        info.setBusinessId(businessId);
        FileInfoVO fileInfoVO = fileInfoRepository.saveAndFlush(info);
        //
        Map<String,Object> map = new HashMap<>();
        map.put("id",String.valueOf(fileInfoVO.getId()));
        map.put("uri",fileInfoVO.getLink());
        map.put("name",fileInfoVO.getName());
        
        return map;
    }

    private String getFileName(String originalFilename) {
        // 获取文件名
        String fileName =originalFilename;
//判断是否为IE浏览器的文件名，IE浏览器下文件名会带有盘符信息
// Check for Unix-style path
        int unixSep = fileName.lastIndexOf('/');
// Check for Windows-style path
        int winSep = fileName.lastIndexOf('\\');
// Cut off at latest possible point
        int pos = (winSep > unixSep ? winSep : unixSep);
        if (pos != -1)  {
            // Any sort of path separator found...
            fileName = fileName.substring(pos + 1);
        }
        return fileName;
    }


    /**
     * 缩略图上传操作
     * @param file
     */
    public Map<String,Object> uploadThumbnail(String nailType, MultipartFile file){
        //
        String fileUri = filesUtils.uploadThumbnail(file, nailType);
        if(StringUtils.isBlank(fileUri)){
            throw new RuntimeException("文件上传失败");
        }
        //
        Map<String,Object> map = new HashMap<>();
        map.put("uri",fileUri);
        return map;
    }

}
