package com.unis.sxls.server.untils;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.Claim;
import com.auth0.jwt.interfaces.DecodedJWT;
import org.apache.commons.lang.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Map;

/**
 * 功能描述: 用于获取company服务的当前登录企业用户信息。
 *
 * @Auther: 蒋青茂
 * @Date: 2018/8/16 15:21
 */
@SuppressWarnings("all")
public class CurrentUtils {

    /**
     * 公用秘钥 保存在服务器端
     */
    private static String SECRET = "MySecret";

    /**
     * 从cookie里面获取信息 封装到map集合的key值
     */

    //公共

    /** 当前用户ip */
    public static final String USER_IP = "uip";

    /** 当前用户id */
    public static final String USER_ID = "uid";


    /** 当前用户名 */
    public static final String USER_NAME = "uname";


    //企业用户

    /**
     * 企业子表id
     */
    public static final String QYUSER_COMPANY_ID = "qy_company_id";

    /**
     * 企业名称
     */
    public static final String QYUSER_COMPANY_NAME = "qy_company_name";

    /**
     * 审核通过企业后的企业所属单位
     */
    public static final String QYUSER_COMPANY_DEPID = "qy_company_depId";

    /**
     * 审核通过企业后的企业所属单位名字
     */
    public static final String QYUSER_COMPANY_DEPNAME = "qy_company_depName";

    /**
     * token
     */

    public static final String TOKEN = "TOKENB";


    public static String getUserInfo(String key){
            String token = getCookie(TOKEN);
            try {
                JWTVerifier verifier = JWT.require(Algorithm.HMAC256(SECRET)).build();
                DecodedJWT jwt = null;
                //如果token过期会抛异常
                jwt = verifier.verify(token);
                Map<String, Claim> result = jwt.getClaims();
                jwt.getKeyId();
                jwt.getToken();
                switch (key) {
                    case USER_IP:
                        return jwt.getClaim(USER_IP).asString();
                    case USER_ID:
                        return jwt.getClaim(USER_ID).asString();
                    case USER_NAME:
                        return jwt.getClaim(USER_NAME).asString();
                    //企业
                    case QYUSER_COMPANY_ID:
                        return jwt.getClaim(QYUSER_COMPANY_ID).asString();
                    case QYUSER_COMPANY_NAME:
                        return jwt.getClaim(QYUSER_COMPANY_NAME).asString();
                    case QYUSER_COMPANY_DEPID:
                        return jwt.getClaim(QYUSER_COMPANY_DEPID).asString();
                    case QYUSER_COMPANY_DEPNAME:
                        return jwt.getClaim(QYUSER_COMPANY_DEPNAME).asString();
                    default:
                        return null;
                }
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
    }

    public static String getCookie(String name){
        HttpServletRequest request = getRequest();
        if (request == null){
            return null;
        }
        Cookie cookies[] = request.getCookies();
        for (Cookie cookie : cookies) {
            if (cookie.getName().equals(name)) {
                String value = cookie.getValue();
                if (StringUtils.isNotBlank(value)){
                    return value;
                }
            }
        }
        return null;
    }
    /**
     * @描述 获取 request
     * @author：蒋青茂
     * @date 2018/6/14
     */
    public static HttpServletRequest getRequest() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        return request;
    }

    /**
     * @描述 获取 response
     * @author：蒋青茂
     * @date 2018/6/14
     */
    public  static HttpServletResponse getResponse(){
        HttpServletResponse response = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getResponse();
        return  response;
    }
}
