package com.unis.sxls.server.untils;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.Claim;
import com.auth0.jwt.interfaces.DecodedJWT;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Map;

/**
 * 功能描述: 用于获取company服务的当前登录粮食局用户信息。
 *
 * @Auther: 蒋青茂
 * @Date: 2018/8/16 15:21
 */
@SuppressWarnings("all")
@Log4j2
public class CurrentUtilsLSJ {

    /**
     * 公用秘钥 保存在服务器端
     */
    private static String SECRET = "MySecret";

    //公共

    /** 当前用户ip */
    public static final String USER_IP = "uip";

    /** 当前用户id */
    public static final String USER_ID = "uid";

    /** 当前用户名 */
    public static final String USER_NAME = "uname";

    /**
     * 当前用户数据范围
     */
    public static final String USER_RANGE = "range";

    /**
     * 当前用户部门id
     */
    public static final String USER_DEPID = "udepId";

    /**
     * 当前用户部门名称
     */
    public static final String USER_DEPNAME = "udepName";

    /**
     * 当前用户所属单位(部门上级)id
     */
    public static final String USER_DEPPID = "udepPid";

    /**
     * 当前用户所属单位(部门上级)名字
     */
    public static final String USER_DEPPNAME = "udepPname";

    /**
     * 当前用户密级code
     */
    public static final String USER_SECURITYCODE = "userSecurityCode";

    /**
     * 当前用户密级名字
     */
    public static final String USER_SECURITYVALUE = "userSecurityValue";

    /**
     * token
     */
    public static final String TOKEN = "TOKENL";
    
    public static final String TOKENB = "TOKENB";
    
    public static String getUserInfo(String key){

        String token = null;
        try {
            token = getCookie(TOKEN);
            if(token==null||token.equals("")) {
            	token = getCookie(TOKENB);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage());
            new RuntimeException("登陆过期请重新登陆！");
        }

        try {
            JWTVerifier verifier = JWT.require(Algorithm.HMAC256(SECRET)).build();
            DecodedJWT jwt = null;
            //如果token过期会抛异常
            jwt = verifier.verify(token);
            Map<String, Claim> result = jwt.getClaims();
            jwt.getKeyId();
            jwt.getToken();
            switch (key) {
                case USER_IP:
                    return jwt.getClaim(USER_IP).asString();
                case USER_ID:
                    return jwt.getClaim(USER_ID).asString();
                case USER_NAME:
                    return jwt.getClaim(USER_NAME).asString();
                case USER_RANGE:
                    return jwt.getClaim(USER_RANGE).asString();
                case USER_DEPID:
                    return jwt.getClaim(USER_DEPID).asString();
                case USER_DEPNAME:
                    return jwt.getClaim(USER_DEPNAME).asString();
                case USER_DEPPID:
                    return jwt.getClaim(USER_DEPPID).asString();
                case USER_DEPPNAME:
                    return jwt.getClaim(USER_DEPPNAME).asString();
                case USER_SECURITYCODE:
                    return jwt.getClaim(USER_SECURITYCODE).asString();
                case USER_SECURITYVALUE:
                    return jwt.getClaim(USER_SECURITYVALUE).asString();
                default:
                    return null;
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage());
            new RuntimeException("当前您的信息异常！请重新登陆。");
            return null;
        }
    }

    public static String getCookie(String name) throws Exception {
        HttpServletRequest request = getRequest();
        if (request == null){
            return null;
        }
        Cookie cookies[] = request.getCookies();
        for (Cookie cookie : cookies) {
            if (cookie.getName().equals(name)) {
                String value = cookie.getValue();
                if (StringUtils.isNotBlank(value)){
                    return value;
                }
            }
        }
        return null;
    }
    /**
     * @描述 获取 request
     * @author：蒋青茂
     * @date 2018/6/14
     */
    public static HttpServletRequest getRequest() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        return request;
    }

    /**
     * @描述 获取 response
     * @author：蒋青茂
     * @date 2018/6/14
     */
    public  static HttpServletResponse getResponse(){
        HttpServletResponse response = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getResponse();
        return  response;
    }
}
