package com.unis.sxls.server.untils;

import com.unis.sxls.server.model.FileInfoVO;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.channels.FileChannel;
import java.util.List;
import java.util.UUID;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * 功能描述：文件上传方法类
 *
 * @author yangy
 */
@Component
public class FilesUtils {

	private final String THUMB_NAIL = "thumbnail";// 缩略图专用存储目录

	/**
	 * /home/ftpadmin/static/
	 */
	@Value("${filesPath}")
	private String filesPath;

	/**
	 * @return java.lang.String
	 * @author yangyh
	 * @Description: 文件上传
	 * @date 2018/6/28 18:01
	 * @update 杨祖榕 2018年11月29日
	 */
	public String upload(MultipartFile multipartFile, String yearMonth) {
		// 文件上传目录uploadDir：基目录(filesPath)/文件类型/上传者
		// 文件上传后路径filePath：文件上传目录/文件名
		if (multipartFile.isEmpty()) {
			return "文件为空。";
		}

		try {
			String originalFilename = multipartFile.getOriginalFilename();// 文件名
			String fileType = getFileType(originalFilename);// 文件类型

			String fileUri = fileType;

			if (StringUtils.isNotBlank(yearMonth)) {
				fileUri = fileUri + File.separator + yearMonth;
			}

			String uploadDirStr = filesPath + File.separator + fileUri;
			File uploadDir = new File(uploadDirStr);
			if (!uploadDir.exists()) {
				uploadDir.mkdirs();
			}

			String fileSuffix = getFileSuffix(originalFilename);
			String filename = getUniqueFilename(fileSuffix);// 新文件名
			fileUri = fileUri + File.separator + filename;
			//
			String filePath = uploadDirStr + File.separator + filename;
			File file = new File(filePath);
			multipartFile.transferTo(file);

			return fileUri;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return "";
	}

	/**
	 * @author 杨祖榕
	 * @Description: 缩略图上传
	 * @date 2019年1月18日
	 */
	public String uploadThumbnail(MultipartFile multipartFile, String nailType) {
		// 上传目录uploadDir：基目录(filesPath)/thumbnail/nailType
		// 上传后，文件路径filePath：文件上传目录/文件名

		if (multipartFile.isEmpty()) {
			return "文件为空。";
		}

		try {
			String originalFilename = multipartFile.getOriginalFilename();// 文件名
			String fileType = getFileType(originalFilename);// 文件类型
			String baseDir = THUMB_NAIL + File.separator + nailType;
			//
			String uploadDirStr = filesPath + File.separator + baseDir;// 文件上传目录
			File uploadDir = new File(uploadDirStr);
			if (!uploadDir.exists()) {
				uploadDir.mkdirs();
			}
			//
			String filename = getUniqueFilename(fileType);// 新文件名
			String filePath = uploadDirStr + File.separator + filename;
			File file = new File(filePath);
			multipartFile.transferTo(file);

			String fileUri = baseDir + File.separator + filename;
			return fileUri;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return "";
	}

	/**
	 * 获取文件后缀名
	 * 
	 * @param fileName
	 * @return
	 */
	private String getFileSuffix(String fileName) {
		String fileSuffix = fileName.substring(fileName.lastIndexOf(".") + 1);// 文件后缀
		return fileSuffix;
	}

	private String getFileType(String fileName) {
		String fileSuffix = fileName.substring(fileName.lastIndexOf(".") + 1);// 文件后缀

		String fileType = null;
		if (fileSuffix.equals("jpg") || fileSuffix.equals("png")) {
			fileType = "images";
		} else if (fileSuffix.equals("doc") || fileSuffix.equals("docx") || fileSuffix.equals("txt")) {
			fileType = "text";
		} else if (fileSuffix.equals("xls") || fileSuffix.equals("xlsx")) {
			fileType = "xlsx";
		} else if (fileSuffix.equals("mp4")) {
			fileType = "video";
		} else {
			fileType = "other";
		}

		return fileType;
	}

	/**
	 * 获取全球维一的文件名，包括文件后缀
	 * 
	 * @param fileSuffix
	 * @return
	 */
	private String getUniqueFilename(String fileSuffix) {
		String uuid = UUIDUtils.uuid();
		return uuid + "." + fileSuffix;
	}

	/**
	 * @return java.lang.String
	 * @Author yang yunhai
	 * @Description: 多文件上传
	 * @Date: 时间:2018/6/13 15:32 * @param files
	 */
	public String filesUpload(MultipartFile[] files, String name) {
		// 判断file数组不能为空并且长度大于0
		try {
			if (files != null && files.length > 0) {
				// 循环获取file数组中的文件
				for (int i = 0; i < files.length; i++) {
					MultipartFile file = files[i];
					// 保存文件
					upload(file, name);
				}
			}
			return "file upload success.";
		} catch (Exception e) {
			return "file upload errors.";
		}
	}

	/**
	 * 下载文件
	 * 
	 * @date 2018/8/23
	 * @update 杨祖榕 2018年11月29日
	 */
	public void downloadFile(String originalFilename, String fileUri, HttpServletResponse response) {

		String filePath = filesPath + File.separator + fileUri;
		File file = new File(filePath);
		if (file.exists()) {
			InputStream inputStream = null;
			try {
				setResponse(response, originalFilename, file);

				inputStream = new FileInputStream(file);
				IOUtils.copy(inputStream, response.getOutputStream());
				inputStream.close();
				response.flushBuffer();

			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				try {
					inputStream.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

	}

	/**
	 * @Description: 返回FILE对象
	 * @date 2019年6月12日,上午9:36:38
	 * @author huangshuai
	 * @version v1.0
	 * @param originalFilename
	 * @param fileUri
	 * @return
	 */
	public File download(String originalFilename, String fileUri) {
		String filePath = filesPath + File.separator + fileUri;
		File file = new File(filePath);
		return file;
	}

	/**
	 * 文件删除
	 * 
	 * @param fileUri
	 */
	public void deleteFile(String fileUri) {
		String filePath = filesPath + File.separator + fileUri;
		File file = new File(filePath);
		if (file.exists()) {
			file.delete();
		}
	}

	/**
	 * 设置返回文件的响应头
	 * 
	 * @param response
	 * @param fileName
	 * @param file
	 * @throws IOException
	 */
	private void setResponse(HttpServletResponse response, String fileName, File file) throws IOException {

		response.reset();// 清空response
		response.setContentType("application/octet-stream");
		response.setCharacterEncoding("UTF-8");
		String encodeFileName = java.net.URLEncoder.encode(fileName, "UTF-8");
		response.addHeader("Content-Disposition",
				"attachment; " + "filename=\"" + encodeFileName + "\";filename*=utf-8''" + encodeFileName);
		response.addHeader("Content-Length", "" + file.length());

	}

	public void downloadFileZip(String fileName, List<FileInfoVO> fileList, HttpServletResponse response) {
		String browsePath = filesPath + File.separator;
		exportToBrowser(browsePath,fileName+".zip",fileList,response);

	}

	/**
	 * 导出zip文件，输出到浏览器
	 */
	public static void exportToBrowser(String browsePath,String fileName, List<FileInfoVO> fileList, HttpServletResponse response) {
		try {
			response.setContentType("application/octet-stream");
			response.setCharacterEncoding("UTF-8");
			String encodeFileName = java.net.URLEncoder.encode(fileName, "UTF-8");
			response.addHeader("Content-Disposition",
					"attachment; " + "filename=\"" + encodeFileName + "\";filename*=utf-8''" + encodeFileName);
			doZIP(browsePath,fileList, response);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * 压缩文件
	 * @param browsePath 文件存储服务器路径
	 * @param response
	 * @throws IOException
	 */
	public static void doZIP(String browsePath,List<FileInfoVO> fileList,HttpServletResponse response) throws IOException{
		InputStream input = null;
		//定义压缩输出流
		ZipOutputStream zipOut = null;
		try {
			zipOut = new ZipOutputStream(response.getOutputStream());
			for (FileInfoVO fileInfoVO : fileList) {
				//拼接相对路径成绝对路径
				String filePath = browsePath+fileInfoVO.getLink();
				File file = new File(filePath);
				input = new FileInputStream(file);
				//压缩包中的子条目
				String name = fileInfoVO.getName();
				ZipEntry zipEntry = new ZipEntry(name);
				zipOut.putNextEntry(zipEntry);
				int len = 0;
				byte[] buffer = new byte[1024];
				while ((len = input.read(buffer)) != -1) {
					zipOut.write(buffer, 0, len);
				}
				input.close();
			}
			zipOut.closeEntry();
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if(null != zipOut) {
				try {
					zipOut.close();
				}catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

}
